local fontColor=Turbine.UI.Color(.8,.7,0);
local fontName=Turbine.UI.Lotro.Font.TrajanPro14;

--Library={};
-- Library elements have the structure {ID,Description,X,Y,Category} where X and Y are the resolution and Category is a value from Categories[x][1]
--Categories={};
-- Categories are the user defined categories the Library table, used for filtering

LibraryWindow = class( Turbine.UI.Lotro.Window );

function LibraryWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );

	self:SetSize(400,400);
	self:SetPosition((Turbine.UI.Display:GetWidth()-self:GetWidth())/2, (Turbine.UI.Display:GetHeight()-self:GetHeight())/2);
	self:SetBackColor(Turbine.UI.Color(.1,.1,.1));
	self:SetText("Image Library");

	self.CatCaption=Turbine.UI.Label();
	self.CatCaption:SetParent(self);
	self.CatCaption:SetSize(90,18);
	self.CatCaption:SetPosition(15,35);
	self.CatCaption:SetFont(Turbine.UI.Lotro.Font.TrajanPro16);
	self.CatCaption:SetForeColor(fontColor)
	self.CatCaption:SetText("Category:");

	self.Category=GaranStuff.IRV.DropDownList();
	self.Category:SetParent(self);
	self.Category:SetPosition(self.CatCaption:GetLeft()+self.CatCaption:GetWidth()+2,self.CatCaption.GetTop()-2);
	self.Category:SetSize(self:GetWidth()-(self.CatCaption:GetWidth()+self.CatCaption:GetLeft())-30,20);
	self.Category:SetBorderColor(fontColor);
	self.Category:SetCurrentBackColor(Turbine.UI.Color(.05,.05,.1));
	self.Category:SetBackColor(Turbine.UI.Color(.05,.05,.05));
	self.Category:SetDropRows(8);

	-- poulate the categories
	local index=0;
	for index=1,#Categories do
		self.Category:AddItem(Categories[index][2],Categories[index][1]);
	end

	self.SubCatCaption=Turbine.UI.Label();
	self.SubCatCaption:SetParent(self);
	self.SubCatCaption:SetSize(90,18);
	self.SubCatCaption:SetPosition(15,60);
	self.SubCatCaption:SetFont(Turbine.UI.Lotro.Font.TrajanPro16);
	self.SubCatCaption:SetForeColor(fontColor)
	self.SubCatCaption:SetText("");

	self.SubCategory=GaranStuff.IRV.DropDownList();
	self.SubCategory:SetParent(self);
	self.SubCategory:SetPosition(self.SubCatCaption:GetLeft()+self.SubCatCaption:GetWidth()+2,self.SubCatCaption.GetTop()-2);
	self.SubCategory:SetSize(self:GetWidth()-(self.SubCatCaption:GetWidth()+self.SubCatCaption:GetLeft())-30,20);
	self.SubCategory:SetBorderColor(fontColor);
	self.SubCategory:SetCurrentBackColor(Turbine.UI.Color(.05,.05,.1));
	self.SubCategory:SetBackColor(Turbine.UI.Color(.05,.05,.05));
	self.SubCategory:SetDropRows(8);
	self.SubCategory:SetVisible(false);

	self.Category.SelectedIndexChanged = function()
		local catID=self.Category:GetValue();
		local catIndex=1;
		local found=false;
		local subCatID=0;
		while catIndex<=#Categories and not found do
			if Categories[catIndex][1]==catID then
				subCatID=Categories[catIndex][3];
				found=true;
			else
				catIndex=catIndex+1;
			end
		end
		if subCatID~=nil and subCatID>0 then
			-- if subcategories are available, refresh them
			while self.SubCategory.ListData:GetItemCount()>0 do
				self.SubCategory:RemoveItemAt(1);
			end
			self.SubCatCaption:SetText(SubCategoryCaption[subCatID]..":");
			for catIndex=1,#SubCategories[subCatID] do
				self.SubCategory:AddItem(SubCategories[subCatID][catIndex][2],SubCategories[subCatID][catIndex][1]);
			end
			self.SubCategory:SetVisible(true);
			self:ListCategoryImages(self.Category:GetValue(),self.SubCategory:GetValue())
		else
			self.SubCatCaption:SetText("");
			self.SubCategory:SetVisible(false);
			self:ListCategoryImages(self.Category:GetValue())
		end
	end

	self.SubCategory.SelectedIndexChanged = function()
			self:ListCategoryImages(self.Category:GetValue(),self.SubCategory:GetValue())
	end

	self.ImageListPanel=Turbine.UI.Label();
	self.ImageListPanel:SetParent(self);
	self.ImageListPanel:SetBackColor(fontColor);
	self.ImageListPanel:SetSize(self:GetWidth()-30,self:GetHeight()-115);
	self.ImageListPanel:SetPosition(15,85);
	self.ImageListPanel.ImageList=Turbine.UI.ListBox();
	self.ImageListPanel.ImageList:SetParent(self.ImageListPanel);
	self.ImageListPanel.ImageList:SetSize(self.ImageListPanel:GetWidth()-2,self.ImageListPanel:GetHeight()-2);
	self.ImageListPanel.ImageList:SetPosition(1,1)
	self.ImageListPanel.ImageList:SetBackColor(Turbine.UI.Color(0,0,0));
	self.ImageListPanel.VScrollBar=Turbine.UI.Lotro.ScrollBar();
	self.ImageListPanel.VScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.ImageListPanel.VScrollBar:SetParent(self.ImageListPanel.ImageList);
	self.ImageListPanel.VScrollBar:SetBackColor(Turbine.UI.Color(0,0,0));
	self.ImageListPanel.VScrollBar:SetWidth(10);
	self.ImageListPanel.VScrollBar:SetHeight(self.ImageListPanel.ImageList:GetHeight());
	self.ImageListPanel.VScrollBar:SetPosition(self.ImageListPanel.ImageList:GetWidth()-10,0);
	self.ImageListPanel.VScrollBar:SetVisible(false);
	self.ImageListPanel.ImageList:SetVerticalScrollBar(self.ImageListPanel.VScrollBar);

	self.RemovalConfirmationPanel=Turbine.UI.Label();
	self.RemovalConfirmationPanel:SetParent(self);
	self.RemovalConfirmationPanel:SetBackColor(Turbine.UI.Color(.7,0,0));
	self.RemovalConfirmationPanel:SetSize(200,100);
	self.RemovalConfirmationPanel:SetPosition((self:GetWidth()-200)/2,(self:GetHeight()-200)/2);
	self.RemovalMsg=Turbine.UI.Label();
	self.RemovalMsg:SetParent(self.RemovalConfirmationPanel);
	self.RemovalMsg:SetSize(self.RemovalConfirmationPanel:GetWidth()-2,self.RemovalConfirmationPanel:GetHeight()-22);
	self.RemovalMsg:SetPosition(1,1);
	self.RemovalMsg:SetBackColor(Turbine.UI.Color(.1,.1,.1));
	self.RemovalMsg:SetForeColor(Turbine.UI.Color(1,0,0));
	self.RemovalMsg:SetTextAlignment( Turbine.UI.ContentAlignment.TopCenter);
	self.RemovalMsg:SetFont(Turbine.UI.Lotro.Font.TrajanPro18);
	self.RemovalMsg:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.RemovalMsg:SetOutlineColor(Turbine.UI.Color(.2,0,0));
	self.RemovalMsg:SetText("Are you sure you wish to remove this image from the Image Library?");
	self.RemoveOK=Turbine.UI.Lotro.Button();
	self.RemoveOK:SetParent(self.RemovalConfirmationPanel);
	self.RemoveOK:SetSize(75,20);
	self.RemoveOK:SetPosition(10,self.RemovalConfirmationPanel:GetHeight()-21)
	self.RemoveOK:SetText("OK");
	self.RemoveOK.Click=function()
		self:RemoveItem();
		self.RemovalConfirmationPanel:SetVisible(false);
	end
	self.RemoveCancel=Turbine.UI.Lotro.Button();
	self.RemoveCancel:SetParent(self.RemovalConfirmationPanel);
	self.RemoveCancel:SetSize(75,20);
	self.RemoveCancel:SetPosition(self.RemovalConfirmationPanel:GetWidth()-85,self.RemovalConfirmationPanel:GetHeight()-21)
	self.RemoveCancel:SetText("Cancel");
	self.RemoveCancel.Click=function()
		self.RemovalConfirmationPanel:SetVisible(false);
	end

	self.RemovalConfirmationPanel:SetVisible(false);

	self.SubCategory:SetZOrder(self.ImageListPanel:GetZOrder()+1);
	self.Category:SetZOrder(self.SubCategory:GetZOrder()+1);
	self.RemovalConfirmationPanel:SetZOrder(self.Category:GetZOrder()+1);

	self.VisibleChanged=function()
		if not self:IsVisible() then
			-- just wanna close the drop down list if it's open. looks weird when it shows up already open when we come back
			self.Category:HideList();
		end
	end
	self.Category:SelectedIndexChanged();
--	self:ListCategoryImages(favWindow.Category:GetValue());
end

function LibraryWindow:ListCategoryImages(categoryID, subCategoryID)
	-- list all of the images for categoryID
	-- note, Library is already sorted by CategoryID and then Name
	local index=1;
	local done=false;
	-- clear the current list
	while self.ImageListPanel.ImageList:GetItemCount()>0 do
		self.ImageListPanel.ImageList:GetItem(1).imageName.MouseClick=nil; -- have to manually remove event handlers
		self.ImageListPanel.ImageList:GetItem(1).imageSize.MouseClick=nil;
		self.ImageListPanel.ImageList:RemoveItemAt(1);
	end
	while index<=#Library and not done do
		if Library[index][5]>categoryID then
			done=true;
		else
			if Library[index][5]==categoryID and (subCategoryID==0 or Library[index][6]==subCategoryID) then
				local tmpEntry=Turbine.UI.Label();
				tmpEntry:SetParent(self.ImageListPanel.ImageList);
				tmpEntry:SetSize(self.ImageListPanel.ImageList:GetWidth()-10,18);
				tmpEntry:SetBackColor(Turbine.UI.Color(.05,.05,.1));
				tmpEntry.imageName=Turbine.UI.Label();
				tmpEntry.imageName:SetParent(tmpEntry);
				tmpEntry.imageName:SetSize(tmpEntry:GetWidth()-71,18);
				tmpEntry.imageName:SetBackColor(Turbine.UI.Color(0,0,0));
				tmpEntry.imageName:SetForeColor(fontColor);
				tmpEntry.imageName:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
				tmpEntry.imageName:SetText(Library[index][4]);
				tmpEntry.imageResourceID=Library[index][1];
				tmpEntry.imageLibraryIndex=index;
				tmpEntry.imageName.MouseClick=function(sender,args)
					if args.Button==1 then
						-- left click
						irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,tonumber(tmpEntry.imageResourceID)));
						irvWindow.ShowGrid=false;
						irvWindow.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
						irvWindow:ShowBackgrounds(self,false);
						self:SetVisible(false);
					else
						-- right click
						self.popUpMenu:GetItems():Get(1):SetText("Resource:"..string.format(irvWindow.numberFormat,tonumber(tmpEntry.imageResourceID)));
						self.popUpMenu:GetItems():Get(2).Click=function()
							self:EditItem(tonumber(tmpEntry.imageResourceID),tmpEntry.imageLibraryIndex);
						end
						self.popUpMenu:GetItems():Get(3).Click=function()
							self:ConfirmRemoveItem(tonumber(tmpEntry.imageResourceID));
						end
						self.popUpMenu:ShowMenu();
					end
				end
				tmpEntry.imageSize=Turbine.UI.Label();
				tmpEntry.imageSize:SetParent(tmpEntry);
				tmpEntry.imageSize:SetSize(70,18);
				tmpEntry.imageSize:SetLeft(tmpEntry.imageName:GetWidth()+1);
				tmpEntry.imageSize:SetBackColor(Turbine.UI.Color(0,0,0));
				tmpEntry.imageSize:SetForeColor(fontColor);
				tmpEntry.imageSize:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
				tmpEntry.imageSize:SetText(Library[index][2].." X "..Library[index][3]);
				tmpEntry.imageSize.MouseClick=function(sender,args)
					if args.Button==1 then
						-- left click
						irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,tonumber(tmpEntry.imageResourceID)));
						irvWindow.ShowGrid=false;
						irvWindow.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
						irvWindow:ShowBackgrounds(self,false);
						self:SetVisible(false);
					else
						-- right click
						self.popUpMenu:GetItems():Get(1):SetText("Resource:"..string.format(irvWindow.numberFormat,tonumber(tmpEntry.imageResourceID)));
						self.popUpMenu:GetItems():Get(2).Click=function()
							self:EditItem(tonumber(tmpEntry.imageResourceID),tmpEntry.imageLibraryIndex);
						end
						self.popUpMenu:GetItems():Get(3).Click=function()
							self:ConfirmRemoveItem(tonumber(tmpEntry.imageResourceID));
						end
						self.popUpMenu:ShowMenu();
					end
				end
				self.ImageListPanel.ImageList:AddItem(tmpEntry);
			end
			index=index+1;
		end
	end
	self.popUpMenu=Turbine.UI.ContextMenu();
	self.popUpMenu:GetItems():Add(Turbine.UI.MenuItem("Item"));
	self.popUpMenu:GetItems():Get(1):SetEnabled(false);
	self.popUpMenu:GetItems():Add(Turbine.UI.MenuItem("Edit"));
	self.popUpMenu:GetItems():Add(Turbine.UI.MenuItem("Remove"));
end

function LibraryWindow:EditItem(itemResourceID,itemLibraryIndex)
	-- need to pop up the Library edit window and populate it with the specified item info
	editLibWindow:SetResource(itemResourceID,itemLibraryIndex);
	editLibWindow:SetVisible(true);

end
function LibraryWindow:ConfirmRemoveItem(itemResourceID)
	self.removeResourceID=itemResourceID;
	self.RemovalConfirmationPanel:SetVisible(true);
end
function LibraryWindow:RemoveItem()
	-- the item ID was previously stored in self.removeResourceID
	local resourceID=tonumber(self.removeResourceID)
	if resourceID~=nil then
		-- find the resource in the Library array
		local index=1;
		local found=false;
		while index<=#Library and not found do
			if Library[index][1]==resourceID then
				table.remove(Library,index);
				found=true;
			else
				index=index+1;
			end
		end
		-- refresh the list
		self:ListCategoryImages(favWindow.Category:GetValue());
	end
end
