-- Image Resource Viewer - part of the GRIM (Graphic Resource Identity Mapping) project
--
-- ability to filter Library list by width and height?
-- need category maintenance window
-- do we want to track "linked images" such as the shadow and quality masks for items?
-- -- or should we just dump that into the description of the shadow and masks? (probably makes more sense to just put it in description)
-- -- that linking info will only be available from the database anyway, not likely anyone is going to match up those resources manually, THAT would be a daunting task...

-- disable controls during "attempts" to display unknown images

-- for images in "Danger" range, display a special danger icon
-- for images not in "KnownGood" range, display a question mark

-- in single image mode, add label for notes if the image is in "Library" list
-- -- this may not be practical, not sure how much delay this will add when finding image in Library, right now Library is pretty small, but it could get significantly large

-- need to be able to switch an image from "Danger" to "Unknown" (don't allow switching to "KnownGood" without using the actual "attempt" mechanism)- use a right click popup from the traffic light image
-- need to be able to switch an image from "KnownGood" to "Danger" or "Unknown" - use a right click popup from the traffic light image

-- need to be able to import other peoples ranges...
-- need to be able to back up current data with a timestamp, that way if something goes REALLY wrong, we can revert to the backup (load data from back up and perform normal save to replace the "bad" data). need to store the backup file name in a list of backups to allow recovery and removal of extra backups
-- considering how time consuming the image identification process can currently be, losing any known image info would be really painful
-- -- on load, look for import file
-- -- -- if found, read all the ranges
-- -- -- see if any part of the range conflicts with an existing range (i.e. range listed as KnownGood but the current user has some of that range flagged as "Danger")
-- -- -- -- if any of the range conflicts, throw up a warning and ask user if the range should merge only Unknown or should over write the conflicts or abort import
-- -- -- -- -- if the user doesn't abort, track the response with the portion of the range that conflicted, we have to ask and track the seperate response for every overlap
-- -- -- if no abort after reading all ranges,
-- -- -- -- reprocess list, applying ranges using user's choices if there is a conflict
-- -- -- -- only change the range type to "Imported" if it was previously "Default" (or didn't exist, ie was "Unknown" values)
-- -- do we want to provide a button to import files if the plugin is already loaded? should be harmless
-- -- since the file name has to be a fixed value -- we COULD allow the user to type in a file name, but that might just be begging for more trouble than it's worth
-- -- to import multiple files would be a pain if we had to unload each time

-- need to be able to export our ranges
-- -- only export ranges listed as "C" (for Custom) to cut down on overlaps


import "Turbine.Debug";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine";
import "Turbine.Utils";
import "Turbine.Gameplay";
import "GaranStuff.IRV";

local colorWhite=Turbine.UI.Color(1,1,1);
local colorBlack=Turbine.UI.Color(0,0,0);
local fontColor=Turbine.UI.Color(.9,.9,.1);
local fontName=Turbine.UI.Lotro.Font.TrajanPro14;

ImageRanges={};
-- ImageRange elements have the structure {Start,End,Class,Type} where Class is "D" or "S" for Danger/Safe and Type is one of "D","I","C" for Default, Imported and Custom respectively
Library={};
-- Library elements have the structure {ID,Description,X,Y,Category} where X and Y are the resolution and Category is a value from Categories[x][1]
Categories={};
-- Categories are the user defined categories for Library, used for filtering
SubCategories={};
SubCategoryCaption={};

function UnloadPlugin()
	SaveData(false);
	-- remove shell commands
	Turbine.Shell.RemoveCommand(irvWindow.shellCmd)
	Turbine.Shell.WriteLine("IRV "..Plugins["IRV"]:GetVersion().." by Garan unloaded");
end

function SaveData(triggerSaveEvent)
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_ImageRanges", ImageRanges );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_Library", Library );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_Categories", Categories );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_SubCategories", SubCategories );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_SubCategoryCaption", SubCategoryCaption );
	local Settings={};
	local resourceID=tonumber(irvWindow.resourceID:GetText());
	if resourceID==nil then resourceID="" end;
	Settings[1]={"ResourceID",resourceID};
	Settings[2]={"NumberFormat",irvWindow.numberFormat};
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_Settings", Settings );

	if triggerSaveEvent then
		Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_TestID", irvWindow.TestID,SaveComplete());
	else
		Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_TestID", irvWindow.TestID);
	end
end

function LoadData()
	ImageRanges=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_ImageRanges");
	Library=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_Library");
	Categories=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_Categories");
	SubCategories=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_SubCategories");
	SubCategoryCaption=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_SubCategoryCaption");
	local testID=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_TestID");
	if ImageRanges==nil then ImageRanges={} end;
	if Library==nil then Library={} end;
	if Categories==nil then Categories={} end;
	if SubCategories==nil then SubCategories={} end;
	if SubCategoryCaption==nil then SubCategoryCaption={} end;
	if testID~=nil then
		-- we were previously testing an "Unknown" ranged resource and it blew up
		-- lock it out to prevent future blowups
		UpdateRange(testID,2);
	end
end

Window = class( Turbine.UI.Window );

function Window:Constructor()
	Turbine.UI.Window.Constructor( self );
	self.Loading=true;
	self:SetWantsUpdates( true );
	self.ShowGrid=false;
	self.delayTime=0;
	self.displayingWarning=false;
	self.numberFormat="0x%x";

	local displayWidth, displayHeight = Turbine.UI.Display.GetSize();
	local windowWidth, windowHeight = Turbine.UI.Display.GetSize();
	local positionLeft=-1; -- default it to an invalid value so we know if we get a saved value
	local positionTop=-1;
	local Settings=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_Settings");
	local lastResourceID=0x41003000;
	if Settings ~=nil then
		local lIndex;
		for lIndex=1,#Settings do
			if Settings[lIndex][1]=="Width" then
				if Settings[lIndex][2]<windowWidth then
					windowWidth=Settings[lIndex][2];
				end
			end
			if Settings[lIndex][1]=="Height" then
				if Settings[lIndex][2]<windowHeight then
					windowHeight=Settings[lIndex][2];
				end
			end
			if Settings[lIndex][1]=="Left" then
				if Settings[lIndex][2]<positionLeft then
					positionLeft=Settings[lIndex][2];
				end
			end
			if Settings[lIndex][1]=="Top" then
				if Settings[lIndex][2]<positionTop then
					positionTop=Settings[lIndex][2];
				end
			end
			if Settings[lIndex][1]=="NumberFormat" then
				self.numberFormat=Settings[lIndex][2];
			end
			if Settings[lIndex][1]=="ResourceID" then
				lastResourceID=Settings[lIndex][2];
			end
		end
	end
	self:SetSize(windowWidth,windowHeight);
	if positionLeft==-1 then positionLeft=( displayWidth - windowWidth ) / 2 end;
	if positionTop==-1 then positionTop=( displayWidth - windowWidth ) / 2 end;
	self:SetPosition( positionLeft, positionTop);
	self:SetBackColor(Turbine.UI.Color(0,0,0));
	self:SetOpacity(.95);
	self:SetText("Image Resource Viewer");

-- stuff for testing background ids

	self.resourceCaption=Turbine.UI.Label();
	self.resourceCaption:SetParent(self);
	self.resourceCaption:SetSize(90,18);
	self.resourceCaption:SetPosition(0,0);
	self.resourceCaption:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.resourceCaption:SetText("Resource ID:");

	self.resourceID=Turbine.UI.Lotro.TextBox();
	self.resourceID:SetParent(self);
	self.resourceID:SetPosition(self.resourceCaption:GetLeft()+self.resourceCaption:GetWidth()+5,self.resourceCaption:GetTop()-2)
	self.resourceID:SetSize(100,18);
	self.resourceID:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.resourceID:SetBackColor(Turbine.UI.Color(.1,.1,.5));
	self.resourceID:SetForeColor(colorWhite);
	self.resourceID:SetText(string.format(self.numberFormat,lastResourceID));

	self.imageSize=Turbine.UI.Label();
	self.imageSize:SetParent(self);
	self.imageSize:SetSize(70,18);
	self.imageSize:SetPosition(self.resourceID:GetLeft()+self.resourceID:GetWidth()+5,self.resourceCaption:GetTop());
	self.imageSize:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);

	self.imageRangeIndicator=Turbine.UI.Label();
	self.imageRangeIndicator:SetParent(self);
	self.imageRangeIndicator:SetSize(9,20);
	self.imageRangeIndicator:SetPosition(self.imageSize:GetLeft()+self.imageSize:GetWidth()+5,self.resourceCaption:GetTop())

	self.imageSolo=Turbine.UI.Label();
	self.imageSolo:SetParent(self);
	self.imageSolo:SetVisible(false)
	self.imageSolo:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.imageSolo.Status=0;
	self.imageSolo.MouseClick=function(sender,args)
		if args.Button==2 then
			if self.imageSolo.Status==1 then
				self.imageMenu:GetItems():Get(1).Click=function()
					editLibWindow:SetResource(tonumber(self.resourceID:GetText()),nil,self.imageWidth,self.imageHeight);
					editLibWindow:SetVisible(true);
				end
				self.imageMenu:ShowMenu();
			end
		end
	end
	self.imageGrid={};

	self.UnknownImageWarning=Turbine.UI.Label();
	self.UnknownImageWarning:SetParent(self);
	self.UnknownImageWarning:SetSize(400,300);
	self.UnknownImageWarning:SetPosition((self:GetWidth()-400)/2,(self:GetHeight()-300)/2);
	self.UnknownImageWarning:SetFont(Turbine.UI.Lotro.Font.TrajanPro20);
	self.UnknownImageWarning:SetBackColor(Turbine.UI.Color(.1,.1,.1));
	self.UnknownImageWarning:SetForeColor(Turbine.UI.Color(1,0,0));
	self.UnknownImageWarning:SetTextAlignment( Turbine.UI.ContentAlignment.TopCenter);
	self.UnknownImageWarning:SetText("You are attempting to display a previously unverified image.  The system is currently saving all data in case the image causes a client crash.  As soon as the current data is saved, the image will be displayed.  If the client does crash, this image will automatically be locked out from displaying in the future (to avoid additional crashes). If the image loads without incident, the image will be flagged as good for future displays.");
	self.UnknownImageWarning:SetVisible(false);
	self.UnknownImageWarning.status=Turbine.UI.Label();
	self.UnknownImageWarning.status:SetParent(self.UnknownImageWarning);
	self.UnknownImageWarning.status:SetSize(0,8);
	self.UnknownImageWarning.status:SetPosition(10,self.UnknownImageWarning:GetHeight()-34);
	self.UnknownImageWarning.status:SetBackColor(Turbine.UI.Color(1,1,0));
	self.UnknownImageWarning.statusBack=Turbine.UI.Label();
	self.UnknownImageWarning.statusBack:SetParent(self.UnknownImageWarning);
	self.UnknownImageWarning.statusBack:SetSize(self.UnknownImageWarning:GetWidth()-20,10);
	self.UnknownImageWarning.statusBack:SetPosition(10,self.UnknownImageWarning:GetHeight()-35);
	self.UnknownImageWarning.statusBack:SetBackColor(Turbine.UI.Color(1,0,0));
	self.UnknownImageWarning.status:SetZOrder(self.UnknownImageWarning.statusBack:GetZOrder()+1);
	self.UnknownImageWarning.btnSkip=Turbine.UI.Lotro.Button();
	self.UnknownImageWarning.btnSkip:SetParent(self.UnknownImageWarning);
	self.UnknownImageWarning.btnSkip:SetSize(100,20);
	self.UnknownImageWarning.btnSkip:SetPosition(15,self.UnknownImageWarning:GetHeight()-22);
	self.UnknownImageWarning.btnSkip:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.UnknownImageWarning.btnSkip.Click=function(sender,args)
		self.delayTime=Turbine.Engine:GetGameTime();
	end
	self.UnknownImageWarning.btnSkip:SetText("Skip Save");

	self.UnknownImageWarning.btnAbort=Turbine.UI.Lotro.Button();
	self.UnknownImageWarning.btnAbort:SetParent(self.UnknownImageWarning);
	self.UnknownImageWarning.btnAbort:SetSize(100,20);
	self.UnknownImageWarning.btnAbort:SetPosition(self.UnknownImageWarning:GetWidth()-self.UnknownImageWarning.btnAbort:GetWidth()-15,self.UnknownImageWarning:GetHeight()-22);
	self.UnknownImageWarning.btnAbort:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.UnknownImageWarning.btnAbort.Click=function(sender,args)
		self.displayingWarning=false;
		self:SetWantsUpdates(false);
		self.UnknownImageWarning:SetVisible(false);
	end
	self.UnknownImageWarning.btnAbort:SetText("Abort");


	local width,height=120,120;
	local gridSpace=6;
	self.GridRows=math.floor(748/(height+gridSpace));
	self.GridCols=math.floor(1024/(width+gridSpace));
	local gridLeft=(windowWidth-width*self.GridCols-gridSpace*(self.GridCols-1))/2
	local gridTop=(windowHeight-20-height*(self.GridRows)-gridSpace*(self.GridRows-1))/2

	for iRow=1, self.GridRows do
		self.imageGrid[iRow]={}
		for iCol=1, self.GridCols do
			self.imageGrid[iRow][iCol]=Turbine.UI.Label();
			self.imageGrid[iRow][iCol].Status=0;
			self.imageGrid[iRow][iCol].imageX=0;
			self.imageGrid[iRow][iCol].imageY=0;
			self.imageGrid[iRow][iCol]:SetParent(self);
			self.imageGrid[iRow][iCol]:SetSize(width,height);
			self.imageGrid[iRow][iCol]:SetPosition(gridLeft+(iCol-1)*(width+gridSpace),gridTop+(iRow-1)*(height+gridSpace));
			self.imageGrid[iRow][iCol]:SetBackColor(Turbine.UI.Color(.1,.1,.1));
			self.imageGrid[iRow][iCol].imageRangeIndicator=Turbine.UI.Label();
			self.imageGrid[iRow][iCol].imageRangeIndicator:SetParent(self.imageGrid[iRow][iCol])
			self.imageGrid[iRow][iCol].imageRangeIndicator:SetSize(9,20);
			self.imageGrid[iRow][iCol].imageRangeIndicator:SetPosition(width-10,7);
			self.imageGrid[iRow][iCol].MouseClick=function()
				self.resourceID:SetText(self.imageGrid[iRow][iCol].imageID:GetText())
				self.ShowGrid=false;
				self.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
				self:ShowBackgrounds(self,false);
			end

			self.imageGrid[iRow][iCol].image=Turbine.UI.Label();
			self.imageGrid[iRow][iCol].image:SetParent(self.imageGrid[iRow][iCol])
			self.imageGrid[iRow][iCol].image:SetSize(32,32);
			self.imageGrid[iRow][iCol].image:SetPosition((width-32)/2,1);
			self.imageGrid[iRow][iCol].image:SetBlendMode(Turbine.UI.BlendMode.Overlay);
			self.imageGrid[iRow][iCol].image.MouseClick=function(sender,args)
				if args.Button==1 then
					self.resourceID:SetText(self.imageGrid[iRow][iCol].imageID:GetText())
					self.ShowGrid=false;
					self.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
					self:ShowBackgrounds(self,false);
				else
					if self.imageGrid[iRow][iCol].Status==1 then
						self.imageMenu:GetItems():Get(1).Click=function()
--Turbine.Shell.WriteLine("itemResourceID:"..itemResourceID,nil,self.imageGrid[iRow][iCol].imageX,self.imageGrid[iRow][iCol].imageY)
							editLibWindow:SetResource(tonumber(self.imageGrid[iRow][iCol].imageID:GetText()),nil,self.imageGrid[iRow][iCol].imageX,self.imageGrid[iRow][iCol].imageY);
							editLibWindow:SetVisible(true);
						end
						self.imageMenu:ShowMenu();
					end
				end
			end

			self.imageGrid[iRow][iCol].imageID=Turbine.UI.Label();
			self.imageGrid[iRow][iCol].imageID:SetParent(self.imageGrid[iRow][iCol])
			self.imageGrid[iRow][iCol].imageID:SetSize(width,14);
			self.imageGrid[iRow][iCol].imageID:SetPosition(0,height-28);
			self.imageGrid[iRow][iCol].imageID:SetFont(Turbine.UI.Lotro.Font.TrajanPro12);
			self.imageGrid[iRow][iCol].imageID:SetBackColor(Turbine.UI.Color(.2,.2,.3));
			self.imageGrid[iRow][iCol].imageID:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);

			self.imageGrid[iRow][iCol].imageSize=Turbine.UI.Label();
			self.imageGrid[iRow][iCol].imageSize:SetParent(self.imageGrid[iRow][iCol])
			self.imageGrid[iRow][iCol].imageSize:SetSize(width,14);
			self.imageGrid[iRow][iCol].imageSize:SetPosition(0,height-14);
			self.imageGrid[iRow][iCol].imageSize:SetFont(Turbine.UI.Lotro.Font.TrajanPro12);
			self.imageGrid[iRow][iCol].imageSize:SetBackColor(Turbine.UI.Color(.2,.2,.3));
			self.imageGrid[iRow][iCol].imageSize:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);
			self.imageGrid[iRow][iCol]:SetVisible(false);
		end
	end

-- create a button to test prev group of backgrounds
	self.btnPriorRange=Turbine.UI.Button();
	self.btnPriorRange:SetParent(self)
	self.btnPriorRange:SetSize(26,20);
	self.btnPriorRange:SetPosition(self.imageRangeIndicator:GetLeft()+self.imageRangeIndicator:GetWidth()+5,self.resourceCaption:GetTop());
	self.btnPriorRange:SetBackground("GaranStuff/IRV/Resources/PriorRangeButton.jpg")
	self.btnPriorRange.Click=function(sender,args)
		-- get the current range index, find the Prior good range with a lower index
		local range,rangeIndex=GetResourceRange(tonumber(self.resourceID:GetText()))
		rangeIndex=rangeIndex-1;
		while rangeIndex>0 and ImageRanges[rangeIndex][3]=="D" do
			rangeIndex=rangeIndex-1;
		end
		if rangeIndex>0 then
			self.resourceID:SetText(ImageRanges[rangeIndex][2]);
			self:ShowBackgrounds(self,false);
		end
	end

	self.btnPrevIDs=Turbine.UI.Button();
	self.btnPrevIDs:SetParent(self)
	self.btnPrevIDs:SetSize(26,20);
	self.btnPrevIDs:SetPosition(self.btnPriorRange:GetLeft()+self.btnPriorRange:GetWidth()+5,self.resourceCaption:GetTop());
	self.btnPrevIDs:SetBackground("GaranStuff/IRV/Resources/BlankButton.jpg")
	self.btnPrevIDs.Image=Turbine.UI.Label();
	self.btnPrevIDs.Image:SetParent(self.btnPrevIDs);
	self.btnPrevIDs.Image:SetSize(16,16);
	self.btnPrevIDs.Image:SetPosition((self.btnPrevIDs:GetWidth()-16)/2,(self.btnPrevIDs:GetHeight()-16)/2)
	self.btnPrevIDs.Image:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.btnPrevIDs.Image:SetBackground(0x41007e0f);
	self.btnPrevIDs.Image:SetMouseVisible(false);
	self.btnPrevIDs.Click=function(sender,args)
		if self.ShowGrid==true then
			-- decrement self.resourceID
			self.resourceID:SetText(string.format(self.numberFormat,tonumber(self.resourceID:GetText())-self.GridRows*self.GridCols));
		else
			self.resourceID:SetText(string.format(self.numberFormat,tonumber(self.resourceID:GetText())-1));
		end
		self:ShowBackgrounds(self,false);
	end

	self.btnGoToImage=Turbine.UI.Button();
	self.btnGoToImage:SetParent(self)
	self.btnGoToImage:SetSize(26,20);
	self.btnGoToImage:SetPosition(self.btnPrevIDs:GetLeft()+self.btnPrevIDs:GetWidth()+5,self.resourceCaption:GetTop());
	self.btnGoToImage:SetBackground("GaranStuff/IRV/Resources/BlankButton.jpg")
	self.btnGoToImage.Image=Turbine.UI.Label();
	self.btnGoToImage.Image:SetParent(self.btnGoToImage);
	self.btnGoToImage.Image:SetSize(18,18);
	self.btnGoToImage.Image:SetPosition((self.btnGoToImage:GetWidth()-18)/2,(self.btnGoToImage:GetHeight()-18)/2)
	self.btnGoToImage.Image:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.btnGoToImage.Image:SetBackground(0x41007dd4);
	self.btnGoToImage.Image:SetMouseVisible(false);
	self.btnGoToImage.Click=function(sender,args)
		self:ShowBackgrounds(self,false);
	end

	self.btnNextIDs=Turbine.UI.Button();
	self.btnNextIDs:SetParent(self)
	self.btnNextIDs:SetSize(26,20);
	self.btnNextIDs:SetPosition(self.btnGoToImage:GetLeft()+self.btnGoToImage:GetWidth()+5,self.resourceCaption:GetTop());
	self.btnNextIDs:SetBackground("GaranStuff/IRV/Resources/BlankButton.jpg")
	self.btnNextIDs.Image=Turbine.UI.Label();
	self.btnNextIDs.Image:SetParent(self.btnNextIDs);
	self.btnNextIDs.Image:SetSize(16,16);
	self.btnNextIDs.Image:SetPosition((self.btnNextIDs:GetWidth()-16)/2,(self.btnNextIDs:GetHeight()-16)/2)
	self.btnNextIDs.Image:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.btnNextIDs.Image:SetBackground(0x41007e12);
	self.btnNextIDs.Image:SetMouseVisible(false);
	self.btnNextIDs.Click=function(sender,args)
		if self.ShowGrid==true then
			-- increment self.resourceID
			self.resourceID:SetText(string.format(self.numberFormat,tonumber(self.resourceID:GetText())+self.GridRows*self.GridCols));
		else
			self.resourceID:SetText(string.format(self.numberFormat,tonumber(self.resourceID:GetText())+1));
		end
		self:ShowBackgrounds(self,false);
	end

	self.btnNextRange=Turbine.UI.Button();
	self.btnNextRange:SetParent(self)
	self.btnNextRange:SetSize(26,20);
	self.btnNextRange:SetPosition(self.btnNextIDs:GetLeft()+self.btnNextIDs:GetWidth()+5,self.resourceCaption:GetTop());
	self.btnNextRange:SetBackground("GaranStuff/IRV/Resources/NextRangeButton.jpg")
	self.btnNextRange.Click=function(sender,args)
		-- get the current range index, find the next good range with a lower index
		local range,rangeIndex=GetResourceRange(tonumber(self.resourceID:GetText()))
		rangeIndex=rangeIndex+1;
		while rangeIndex<=#ImageRanges and ImageRanges[rangeIndex][3]=="D" do
			rangeIndex=rangeIndex+1;
		end
		if rangeIndex<=#ImageRanges then
			self.resourceID:SetText(ImageRanges[rangeIndex][1]);
			self:ShowBackgrounds(self,false);
		end
	end

	self.btnForceView=Turbine.UI.Lotro.Button();
	self.btnForceView:SetParent(self)
	self.btnForceView:SetSize(75,20);
	self.btnForceView:SetPosition(self.btnNextRange:GetLeft()+self.btnNextRange:GetWidth()+5,self.resourceCaption:GetTop());
	self.btnForceView:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.btnForceView:SetText("Attempt");
	self.btnForceView:SetVisible(false);
	self.btnForceView.Click=function(sender,args)
		self:ShowBackgrounds(self,true);
	end

-- create grid toggle button to test Next group of backgrounds
	self.btnGridToggle=Turbine.UI.Button();
	self.btnGridToggle:SetParent(self)
	self.btnGridToggle:SetSize(40,20);
	self.btnGridToggle:SetPosition(self.btnForceView:GetLeft()+self.btnForceView:GetWidth()+5,self.resourceCaption:GetTop());
	self.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
	self.btnGridToggle.Click=function(sender,args)
		self.ShowGrid= not self.ShowGrid
		if self.ShowGrid==true then
			self.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleGrid.jpg");
		else
			self.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
		end
		self:ShowBackgrounds(self,false);
	end

	self.btnFormatToggle=Turbine.UI.Button();
	self.btnFormatToggle:SetParent(self)
	self.btnFormatToggle:SetSize(40,20);
	self.btnFormatToggle:SetPosition(self.btnGridToggle:GetLeft()+self.btnGridToggle:GetWidth()+6,self.resourceCaption:GetTop());
	if self.numberFormat=="0x%x" then
		self.btnFormatToggle:SetBackground("GaranStuff/IRV/Resources/ToggleHex.jpg");
	else
		self.btnFormatToggle:SetBackground("GaranStuff/IRV/Resources/ToggleDec.jpg");
	end
	self.btnFormatToggle.Click=function(sender,args)
		if self.numberFormat=="0x%x" then
			self.numberFormat="%d";
			self.btnFormatToggle:SetBackground("GaranStuff/IRV/Resources/ToggleDec.jpg");
		else
			self.numberFormat="0x%x";
			self.btnFormatToggle:SetBackground("GaranStuff/IRV/Resources/ToggleHex.jpg");
		end
		self.resourceID:SetText(string.format(self.numberFormat,self.resourceID:GetText()));
		self:ShowBackgrounds(self,false);
	end

	self.btnSearch=Turbine.UI.Button();
	self.btnSearch:SetParent(self)
	self.btnSearch:SetSize(26,20);
	self.btnSearch:SetPosition(self.btnFormatToggle:GetLeft()+self.btnFormatToggle:GetWidth()+6,self.resourceCaption:GetTop());
	self.btnSearch:SetBackground("GaranStuff/IRV/Resources/SearchButton.jpg");
	self.btnSearch.Click=function(sender,args)
		libWindow:SetVisible(true);
	end

	self.Update = function(sender,args)
		if self.Loading and (Plugins["IRV"] ~= nil) then
			Plugins["IRV"].Unload = function(self,sender,args)
				UnloadPlugin();
			end
			self.Loading=false;
			Turbine.Shell.WriteLine("IRV "..Plugins["IRV"]:GetVersion().." by Garan loaded");
			self:SetWantsUpdates(false);
		end
		if self.displayingWarning then
			local curTime=Turbine.Engine:GetGameTime()
			local delta=1-(self.delayTime-curTime)/30
			if delta>0 then
				self.UnknownImageWarning.status:SetWidth((self.UnknownImageWarning:GetWidth()-20)*delta);
			end

			if self.delayTime<Turbine.Engine:GetGameTime() then
				self:SetWantsUpdates(false);
				SaveComplete2();
			end
		end
	end       

	self.btnSearch=Turbine.UI.Button();
	self.btnSearch:SetParent(self)
	self.btnSearch:SetSize(26,20);
	self.btnSearch:SetPosition(self:GetWidth()-28,self.resourceCaption:GetTop());
	self.btnSearch:SetBackground("GaranStuff/IRV/Resources/CloseButton.jpg");
	self.btnSearch.Click=function(sender,args)
		self:SetVisible(false);
	end

	self.resourceCaption:SetZOrder(self.imageSolo:GetZOrder()+1)
	self.resourceID:SetZOrder(self.resourceCaption:GetZOrder()+1)
	self.imageSize:SetZOrder(self.resourceID:GetZOrder()+1)
	self.imageRangeIndicator:SetZOrder(self.imageSize:GetZOrder()+1)
	self.btnPrevIDs:SetZOrder(self.imageRangeIndicator:GetZOrder()+1)
	self.btnForceView:SetZOrder(self.btnPrevIDs:GetZOrder()+1)
	self.btnNextIDs:SetZOrder(self.btnForceView:GetZOrder()+1)
	self.btnNextRange:SetZOrder(self.btnNextIDs:GetZOrder()+1)
	self.btnGridToggle:SetZOrder(self.btnNextRange:GetZOrder()+1)
	self.btnFormatToggle:SetZOrder(self.btnGridToggle:GetZOrder()+1)
	self.UnknownImageWarning:SetZOrder(self.imageSolo:GetZOrder()+1);

	-- capture Key events
	self.KeyDown=function(sender, args)
--Turbine.Shell.WriteLine("KeyDown Action="..args.Action..", .Alt="..tostring(args.Alt)..", .Control="..tostring(args.Control)..", .Shift="..tostring(args.Shift))
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			sender:SetVisible( false );
		end
	end

	self.VisibleChanged=function()
		if (not self:IsVisible()) then
			-- hide all the other windows when we are hidden
			if libWindow~=nil then
				libWindow:SetVisible(false);
			end
			if editLibWindow~=nil then
				editLibWindow:SetVisible(false);
			end
		end
	end

	-- register shell commands
	self.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("irv;ImageResourceViewer",self.shellCmd);
	self.shellCmd.Execute = function(sender, cmd, args)
		if (cmd == "irv" or cmd=="ImageResourceViewer") then
			if (args=="show") then
				self:SetVisible(true);
			elseif (args=="hide") then
				self:SetVisible(false);
			elseif (args=="toggle") then
				self:SetVisible(not self:IsVisible());
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /irv show");
				Turbine.Shell.WriteLine("       Displays the Image Resource Viewer.");
				Turbine.Shell.WriteLine("  /irv hide");
				Turbine.Shell.WriteLine("       Hides the Image Resource Viewer.");
				Turbine.Shell.WriteLine("  /irv toggle");
				Turbine.Shell.WriteLine("       Toggles the Image Resource Viewer (shows it if it was hidden, hides it if it was showing).");
			end
		end
	end
	self.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /irv show\n       Displays the Image Resource Viewer.\n  /irv hide\n       Hides the Image Resource Viewer.\n  /irv toggle\n       Toggles the Image Resource Viewer (shows it if it was hidden, hides it if it was showing).");
	end

	self.shellCmd.GetShortHelp = function(sender, cmd)
		return("irv - usage: /irv show | hide | toggle");
	end
	self:SetWantsKeyEvents(true);

	self.imageMenu=Turbine.UI.ContextMenu();
	self.imageMenu:GetItems():Add(Turbine.UI.MenuItem("Edit Library Entry"));
end

function GetResourceRange(resource)
	-- determine which range this resource belongs in, Unknown=0, Safe=1, Danger=2
	local range=0;
	local index=1;
	local found=false;
	local passed=false
	while index<=#ImageRanges and not found and not passed do
		if resource>=ImageRanges[index][1] and resource<=ImageRanges[index][2] then
			if ImageRanges[index][3]=="S" then
				range=1;
			else
				range=2;
			end
			found=true;
		else
			if resource<ImageRanges[index][1] then
				passed=true;
			else
				index=index+1;
			end
		end
	end
	return range,index
end
function UpdateRange(resource,class)
	local range,rangeID=GetResourceRange(resource);
	local priorRange,priorRangeID=GetResourceRange(resource-1);
	local nextRange,nextRangeID=GetResourceRange(resource+1);

	if class~=range then
		if range==0 then
			-- see if there's a range below to attach to
			if priorRange==class then
				if nextRange==class then
					-- merge the ranges
					ImageRanges[priorRangeID][2]=ImageRanges[nextRangeID][2];
					ImageRanges[priorRangeID][4]="C"; -- flag the range as Custom
					table.remove(ImageRanges,nextRangeID);
				else
					-- just extend the prior range
					ImageRanges[priorRangeID][2]=resource;
					ImageRanges[priorRangeID][4]="C"; -- flag the range as Custom
				end
			else
				if nextRange==class then
					-- just extend the next range
					ImageRanges[nextRangeID][1]=resource;
					ImageRanges[nextRangeID][4]="C"; -- flag the range as Custom
				else
					-- no prior or next, need a new range
					-- this is an odd one since we need to know the right index to insert
					-- rangeid should point to one past the number of ranges or to the first range that is greater than our resourceid
					if class==1 then
						table.insert(ImageRanges,rangeID,{resource,resource,"S","C"})
					elseif class==2 then
						table.insert(ImageRanges,rangeID,{resource,resource,"D","C"})
					end
				end
			end
		else
			if resource==ImageRanges[rangeID][1] then
				if priorRange~=0 then
					ImageRanges[priorRangeID][2]=resource;
					ImageRanges[priorRangeID][4]="C";
					if ImageRanges[rangeID][2]==resource then
						table.remove(ImageResources,rangeID);
					else
						ImageRanges[rangeID][1]=resource+1;
					end
				else
					if ImageRanges[rangeID][2]==resource then
						if class==1 then
							ImageRanges[rangeID][3]="S";
						elseif class==2 then
							ImageRanges[rangeID][3]="D";
						end
						ImageRanges[rangeID][4]="C";
					else
						ImageRanges[rangeID][1]=resource+1;
						if class==1 then
							table.insert(ImageRanges,rangeID,{resource,resource,"S","C"})
						elseif class==2 then
							table.insert(ImageRanges,rangeID,{resource,resource,"D","C"})
						end
					end
				end
			else

				if resource==ImageRanges[rangeID][2] then
					if nextRange~=0 then
						ImageRanges[nextRangeID][1]=resource;
						if ImageRanges[rangeID][1]==resource then
							table.remove(ImageResources,rangeID);
						else
							ImageRanges[rangeID][2]=resource-1;
						end
						ImageRanges[nextRangeID][4]="C";
					else
						if ImageRanges[rangeID][1]==resource then
							if class==1 then
								ImageRanges[rangeID][3]="S";
							elseif class==2 then
								ImageRanges[rangeID][3]="D";
							end
							ImageRanges[rangeID][4]="C";
						else
							ImageRanges[rangeID][2]=resource-1;
							if class==1 then
								table.insert(ImageRanges,rangeID+1,{resource,resource,"S","C"})
							elseif class==2 then
								table.insert(ImageRanges,rangeID+1,{resource,resource,"D","C"})
							end
						end
					end
				else
					-- need to split our old range
					table.insert(ImageRanges,rangeID+1,ImageRanges[rangeID])
					ImageRanges[rangeID+1][1]=resource+1;
					ImageRanges[rangeID][2]=resource-1;
					if class==1 then
						table.insert(ImageRanges,rangeID+1,{resource,resource,"S","C"})
					elseif class==2 then
						table.insert(ImageRanges,rangeID+1,{resource,resource,"D","C"})
					end
				end
			end
		end
	end
end

function Window:ShowBackgrounds(sender,showUnknown)
	local resource;
	local range;
	self.btnForceView:SetVisible(false);
	if self.ShowGrid==true then
		local iRow,iCol;
		local imageWidth,imageHeight;
		local cellMaxWidth=self.imageGrid[1][1]:GetWidth()-12;
		local cellMaxHeight=self.imageGrid[1][1]:GetHeight()-30;
		self.imageSolo:SetVisible(false);
		self.imageRangeIndicator:SetVisible(false);
		self.imageSize:SetVisible(false);
		for iRow=1, #self.imageGrid do
			for iCol=1,#self.imageGrid[iRow] do
				self.imageGrid[iRow][iCol]:SetVisible(true);
				resource=tonumber(self.resourceID:GetText())+(iRow-1)*#self.imageGrid[1]+(iCol-1) ;
				self.imageGrid[iRow][iCol].imageID:SetText(string.format(self.numberFormat,resource));
				range=GetResourceRange(resource);
				self.imageGrid[iRow][iCol].Status=range;
				if range==2 then
					self.imageGrid[iRow][iCol].imageRangeIndicator:SetBackground("GaranStuff/IRV/Resources/redlight.jpg");
					self.imageGrid[iRow][iCol].image:SetVisible(false);
					self.imageGrid[iRow][iCol].imageSize:SetText("");
-- display message that resource is in a range flagged as invalid
				else
					if range==1 then
						self.imageGrid[iRow][iCol].imageRangeIndicator:SetBackground("GaranStuff/IRV/Resources/greenlight.jpg");
						self.imageGrid[iRow][iCol].image:SetVisible(true);
						self.imageGrid[iRow][iCol].image:SetSize(32,32);
						self.imageGrid[iRow][iCol].image:SetBackground(resource);
						self.imageGrid[iRow][iCol].image:SetStretchMode(2); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
						imageWidth,imageHeight=self.imageGrid[iRow][iCol].image:GetSize();
						self.imageGrid[iRow][iCol].imageX=imageWidth;
						self.imageGrid[iRow][iCol].imageY=imageHeight;
						self.imageGrid[iRow][iCol].imageSize:SetText(imageWidth.." x "..imageHeight);
						self.imageGrid[iRow][iCol].image:SetStretchMode(0); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
						if imageWidth>cellMaxWidth then imageWidth=cellMaxWidth end;
						if imageHeight>cellMaxHeight then imageHeight=cellMaxHeight end;
						self.imageGrid[iRow][iCol].image:SetSize(imageWidth,imageHeight);
						self.imageGrid[iRow][iCol].image:SetPosition((cellMaxWidth-imageWidth)/2+1,(cellMaxHeight-imageHeight)/2+1);
					else
						self.imageGrid[iRow][iCol].imageRangeIndicator:SetBackground("GaranStuff/IRV/Resources/yellowlight.jpg");
						-- we ignore showUnknown in grid mode
						self.imageGrid[iRow][iCol].image:SetVisible(false);
						self.imageGrid[iRow][iCol].imageSize:SetText("");
-- display message that resource is in a range flagged as unkknown
					end
				end
			end
		end
	else
		for iRow=1, #self.imageGrid do
			for iCol=1,#self.imageGrid[iRow] do
				self.imageGrid[iRow][iCol]:SetVisible(false);
			end
		end
		self.imageRangeIndicator:SetVisible(true);
		self.imageSize:SetVisible(true);
		resource=tonumber(self.resourceID:GetText());
		self.resourceID:SetText(string.format(self.numberFormat,resource)); -- refresh the number using the current format
		range=GetResourceRange(resource);
		self.imageSolo.Status=range;
		if range==2 then
			self.imageRangeIndicator:SetBackground("GaranStuff/IRV/Resources/redlight.jpg");
			self.imageSolo:SetVisible(false);
			self.imageSize:SetText("");
-- display message that resource is in a range flagged as invalid
		else
			if range==1 then
				self.imageRangeIndicator:SetBackground("GaranStuff/IRV/Resources/greenlight.jpg");
				self.imageSolo:SetVisible(true);
				self.imageSolo:SetSize(100,100);
				self.imageSolo:SetBackground(resource)
				self.imageSolo:SetStretchMode(2); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
				self.imageWidth,self.imageHeight=self.imageSolo:GetSize();
				self.imageSize:SetText(self.imageWidth.." x "..self.imageHeight);
				self.imageSolo:SetStretchMode(0); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
				self.imageSolo:SetSize(self.imageWidth,self.imageHeight); -- for some reason, we have to manually set the image size even though the width and height are reported as having changed already
				self.imageSolo:SetPosition((self:GetWidth()-self.imageWidth)/2,(self:GetHeight()-self.imageHeight)/2)
			else
				self.btnForceView:SetVisible(true);
				self.imageRangeIndicator:SetBackground("GaranStuff/IRV/Resources/yellowlight.jpg");
				if showUnknown==true then
-- save any changes incase we blow up
-- save the resource number as "bad resource" so we can automatically lock it out on reload
irvWindow.TestID=resource;
-- display the message indicating that we are waiting to save our state before loading image
self.UnknownImageWarning:SetVisible(true);
SaveData(true);
self.delayTime=Turbine.Engine:GetGameTime()+30;
self.displayingWarning=true;
self.UnknownImageWarning.status:SetWidth(0);
self:SetWantsUpdates(true);  -- for some reason, the save function fires the dataSaveComplete event long before the data is actully saved :(
self.btnForceView:SetVisible(false);

-- the SaveComplete event handler will finish displaying the image once the settings are actually saved
--					self.imageSolo:SetVisible(true);
--					self.imageSolo:SetSize(100,100);
--					self.imageSolo:SetBackground(resource)
--					self.imageSolo:SetStretchMode(2); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
--					self.imageWidth,self.imageHeight=self.imageSolo:GetSize();
--					self.imageSize:SetText(self.imageWidth.." x "..self.imageHeight);
--					self.imageSolo:SetStretchMode(0); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
--					self.imageSolo:SetSize(self.imageWidth,self.imageHeight); -- for some reason, we have to manually set the image size even though the width and height are reported as having changed already
--					self.imageSolo:SetPosition((self:GetWidth()-self.imageWidth)/2,(self:GetHeight()-self.imageHeight)/2)
				else
					self.imageSolo:SetVisible(false);
					self.imageSize:SetText("");
				end
-- display message that resource is in a range flagged as unkknown
			end
		end
	end
end
function SaveComplete()
	-- theoretically this handler is called when the data has been completely saved
	-- however, experience shows us that the event fires before the data is really saved to disk, so a client crash will still prevent the data from saving :(
	-- which renders this mechanism pretty much useless
end
function SaveComplete2()
local resource=irvWindow.TestID;
	-- function called when we completed saving data while testing a new image
	irvWindow.UnknownImageWarning:SetVisible(false);
	irvWindow.imageSolo:SetVisible(true);
	irvWindow.imageSolo:SetSize(100,100);
	irvWindow.imageSolo:SetBackground(resource)
	irvWindow.imageSolo:SetStretchMode(2); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
	irvWindow.imageWidth,irvWindow.imageHeight=irvWindow.imageSolo:GetSize();
	irvWindow.imageSize:SetText(irvWindow.imageWidth.." x "..irvWindow.imageHeight);
	irvWindow.imageSolo:SetStretchMode(0); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
	irvWindow.imageSolo:SetSize(irvWindow.imageWidth,irvWindow.imageHeight); -- for some reason, we have to manually set the image size even though the width and height are reported as having changed already
	irvWindow.imageSolo:SetPosition((irvWindow:GetWidth()-irvWindow.imageWidth)/2,(irvWindow:GetHeight()-irvWindow.imageHeight)/2)
	-- update the resource number as OK
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_TestID");
	UpdateRange(resource,1);
	irvWindow.imageRangeIndicator:SetBackground("GaranStuff/IRV/Resources/greenlight.jpg");
	irvWindow.TestID=nil;
end

function DumpImageRangesToChat()
	local index=1
	while index<=#ImageRanges do
		Turbine.Shell.WriteLine("ImageRanges[#ImageRanges+1]={"..string.format("0x%x",ImageRanges[index][1])..","..string.format("0x%x",ImageRanges[index][2])..", \""..ImageRanges[index][3].."\", \"D\"}" )
		index=index+1
	end
end
function DumpLibraryToChat()
	local index=1
	while index<=#Library do
		Turbine.Shell.WriteLine("Library[#Library+1]={"..string.format("0x%x",Library[index][1])..","..tostring(Library[index][2])..", "..tostring(Library[index][3])..", \""..tostring(Library[index][4]).."\", "..tostring(Library[index][5])..", "..tostring(Library[index][6]).."}" )
		index=index+1
	end
end

LoadData(); -- we want to have all of the categories and other arrays filled before we try to instantiate the windows since some controls depend on this data (like the category list)

if #ImageRanges==0 or #Categories==0 or #Library==0 then
import "GaranStuff.IRV.Defaults";
LoadDefaults()
end

-- Instantiate the actual windows, etc

irvWindow=Window();
libWindow=LibraryWindow();
libWindow:SetVisible(false);
editLibWindow=EditLibraryWindow();
editLibWindow:SetVisible(false);
catMaintWindow=CategoryMaintWindow();
catMaintWindow:SetVisible(false);

irvWindow:SetZOrder(irvWindow:GetZOrder()+1); -- force irv to "Topmost"
libWindow:SetZOrder(irvWindow:GetZOrder()+1);
editLibWindow:SetZOrder(libWindow:GetZOrder()+1);
catMaintWindow:SetZOrder(editLibWindow:GetZOrder()+1);

--DumpImageRangesToChat(); -- used to generate the default ranges for builds
--DumpLibraryToChat(); -- used to generate the default ranges for builds

irvWindow:ShowBackgrounds(false);
